package com.myco.plugin

import org.gradle.api.InvalidUserDataException
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.UnknownProjectException
import org.gradle.api.artifacts.Dependency
import org.gradle.api.artifacts.ProjectDependency
import org.gradle.api.logging.Logger
import org.gradle.api.logging.Logging

class CompositePlugin implements Plugin<Project> {
    static final Logger log = Logging.getLogger(CompositePlugin)
    static final String COMPOSITE_EXTENSION_NAME = 'composite'
    static final String DEFAULT_GROUP = "com.myco"

    def CONFIGURATION_CLOSURE = { println "Gradle build for project '${project.name}' version '${project.version}'." }

    @Override
    void apply(Project project) {
        CompositePluginExtension compositePluginExtension = project.extensions.create(COMPOSITE_EXTENSION_NAME, CompositePluginExtension)
        compositePluginExtension.setProject(project)

        if (!project.group) {
            log.debug "Setting group from: '${project.group}' to: '${DEFAULT_GROUP}'."
            project.group = DEFAULT_GROUP
        }

        // Apply base configuration
        project.configure(project, CONFIGURATION_CLOSURE)

        project.afterEvaluate {
            if (project.version.contains("unspecified")) {
                throw new InvalidUserDataException("The project.version must be specified.")
            }
        }
    }

    /**
     * Define a Method for resolving dependencies as projects if possible. Otherwise the passed in dependency notation
     */
    static Dependency resolveDependency(Object dependencyNotation, Project project, String configurationName = null) {
        def forceJars = (project.hasProperty("forceJars") && project.forceJars == "true")
        log.info("resolveDependency forceJars: ${forceJars}, dependencyNotation: ${dependencyNotation}")

        def dependency = project.dependencies.create(dependencyNotation)

        if (!forceJars) {
		
			// TODO: Figure out how to replace this with dependencySubstitution now that we know it exists.
            ProjectDependency projectDependency
            try {
                if (configurationName == null) {
                    projectDependency = project.dependencies.project(path: projectPath)
                } else {
                    projectDependency = project.dependencies.project(path: projectPath, configuration: configurationName)
                }
            } catch (UnknownProjectException e) {
                // No local project, use maven repository for dependency resolution. This is not an error condition but an expected path for this case.
                projectDependency = null
            }

            if (projectDependency != null) {
                if (log.isTraceEnabled()) {
                    projectDependency.getDependencyProject().getTasks().each { log.trace(it) }
                }
                println("Found non-jar project dependency: " + projectDependency)
                return project.dependencies.create(projectDependency)
            }
        }

        return dependency
    }

}
