package com.myco.plugin

import org.gradle.api.Project
import org.gradle.api.artifacts.Dependency

/**
 * Composite plugin API definition. Defines entry point for calling Composite methods in Gradle files (e.g., composite.resolveDependency()).
 */
class CompositePluginExtension {
    private Project project

    public void setProject(Project project) {
        this.project = project
    }

    /**
     * Define a Method for resolving dependencies as projects if possible. Otherwise the passed in dependency notation
     * (group/name/version map) is used to resolve the dependency.
     * @param dependencyNotation The map of the dependency in question, separated properly
     * @param configurationName If the dependency needs a nonstandard configuration, add that here
     */
    Dependency resolveDependency(Object dependencyNotation, String configurationName = null) {
        return CompositePlugin.resolveDependency(dependencyNotation, project, configurationName)
    }
}
